@@ -48,6 +48,8 @@ my $r_stdx = 600; my $r_stdy = 400;
 my $r_minx = 0; my $r_maxx = 0;
 my $r_miny = 0; my $r_maxy = 0;
 my $resolution_on = 0;
+my $force_resolution_chunk = 0;
+
 
 my @p_picno;
 my @p_stdp;
@@ -248,6 +250,28 @@ sub palette_simple_chunk
     end_chunk();
 }
 
+sub adaptive_palette_chunk
+{   my $t = $_[0];
+    begin_chunk("APal", 0, $t);
+    end_chunk();
+}
+
+sub resolution_chunk
+{   my $t = $_[0];
+    begin_chunk("Reso", 0, $t);
+    end_chunk();
+}
+
+sub rectangle_chunk
+{
+    my $t = $_[0];
+    my ($num, $width, $height) = split(" ", $t);
+    begin_chunk("Rect", $num, "");
+    four_word($width);
+    four_word($height);
+    end_chunk();
+}
+
 sub frontispiece_chunk
 {   my $t = $_[0];
     begin_chunk("Fspc", 0, "");
@@ -294,6 +318,7 @@ sub isgraphics
     if ($id eq "JPEG") { return 1; }
     if ($id eq "GIF ") { return 1; }
     if ($id eq "GFX ") { return 1; }
+    if ($id eq "Rect") { return 1; }
     return 0;
 }
 
@@ -389,6 +414,17 @@ sub interpret
     {   release_chunk($1);
         return;
     }
+    if ($command =~ /^\s*adaptive_palette\s+"(.*)"/)
+    {
+	adaptive_palette_chunk($1);
+	return;
+    }
+    if ($command =~ /^\s*resolution_chunk\s+"(.*)"/)
+    {
+	$force_resolution_chunk = 1;
+	resolution_chunk($1);
+	return;
+    }
     if ($command =~ /^\s*resolution\s+(\d*)x(\d*)\s*(.*)$/m)
     {   $r_stdx = $1; $r_stdy = $2;
         $r_minx = 0; $r_maxx = 0;
@@ -519,7 +555,12 @@ sub interpret
 
     # Generate Pict chunks
     if ($command =~ /^\s*picture\s+([a-zA-Z_0-9]*)\s*"(.*)"\s*(.*)$/m ||
-	$command =~ /^\s*cover\s+"(.*)"\s*$/m)
+	$command =~ /^\s*cover\s+"(.*)"\s*$/m ||
+#	$command =~ /^\s*rectangle\s+([a-zA-Z_0-9]*)\s*\d{1,3}\s*\d{1,3}\s*$/m)
+#	$command =~ /^\s*rectangle\s*$/m)
+#	$command =~ /^\s*rectangle\s+(\d{1,3})\s*(\d{1,3})\s*(\d{1,3})\s*$/m)
+#	$command =~ /^\s*rectangle\s+(\d*)\s*(\d*)\s*(\d*)$/m)
+	$command =~ /^\s*rectangle\s+(.*)\s*(.*)\s*(.*)$/m)
     {   my $pnumt = $1;
 	my $pfile = $2;
 	my $rest = $3;
@@ -546,6 +587,13 @@ sub interpret
 	    $ext = ($pfile =~ m/([^.]+)$/)[0];
 	    frontispiece_chunk($pnum);
 	}
+#	elsif ($command =~ /^\s*rectangle\s+(\d*)\s*(\d*)\s*(\d*)$/m)
+#	elsif ($command =~ /^\s*rectangle\s+(\d{1,3})\s*(\d{1,3})\s*(\d{1,3})\s*$/m)
+#	elsif ($command =~ /^rectangle\s*/m)
+	elsif ($command =~ /^\s*rectangle\s+(.*)\s*(.*)\s*(.*)$/m)
+	{
+	    $ext = "rec";
+	}
 	else
         {   $pnum = $next_pnum;
             $next_pnum = $next_pnum + 1;
@@ -569,8 +617,10 @@ sub interpret
 	} elsif ($ext eq "gfx") {
 	    begin_chunk("GFX ", $pnum, $pfile);
 	    end_chunk();
+	} elsif ($ext eq "rec") {
+	    rectangle_chunk($1);
 	} else {
-	    fatal("Unknown picture type");
+	    fatal("Unknown picture type: $command");
 	}
 
         if ($rest =~ /^\s*$/m)
@@ -718,7 +768,7 @@ sub interpret
 
     if (($1 eq "copyright") || ($1 eq "palette") || ($1 eq "picture")
         || ($1 eq "release") || ($1 eq "resolution") || ($1 eq "sound")
-        || ($1 eq "storyfile"))
+        || ($1 eq "storyfile") || ($1 eq "rectangle") || ($1 eq "resolution_chunk") )
     {   error("incorrect syntax for $1 command");
         return;
     }
@@ -802,7 +852,7 @@ if (!$create_blorb) {
     exit 0;
 }
 
-if ($resolution_on == 1)
+if ($resolution_on == 1 && !$force_resolution_chunk)
 {
     begin_chunk("Reso", 0, "");
     four_word($r_stdx);
@@ -864,16 +914,13 @@ four_word($important_count);
 for ($x = 0; $x < $chunk_count; $x = $x + 1)
 {   if ($chunk_important_array[$x] == 1)
     {   $type = $chunk_id_array[$x];
-	if (($type eq "PNG ") || ($type eq "JPEG") || ($type eq "GIF ")
-		|| ($type eq "GFX "))
+	if (isgraphics($type))
 	{   $type = "Pict";
 	}
-        if (($type eq "AIFF") || ($type eq "MOD ") || ($type eq "OGGV")
-		|| ($type eq "MP3 "))
+	if (isaudio($type))
         {   $type = "Snd ";
         }
-	if (($type eq "ZCOD") || ($type eq "GLUL") || ($type eq "MAGS")
-		|| ($type eq "ADRI"))
+	if (isexec($type))
 	{   $type = "Exec";
 	}
 	write_chunk($type);
@@ -889,13 +936,11 @@ for ($x = 0; $x <= $max_resource_num; $x = $x + 1)
 
 for ($x = 0; $x < $chunk_count; $x = $x + 1)
 {   $type = $chunk_id_array[$x];
-    if (($type eq "PNG ") || ($type eq "JPEG") || ($type eq "GIF ")
-		|| ($type eq "GFX "))
+    if (isgraphics($type))
     {   $picture_numbering[$chunk_number_array[$x]] = $x;
         $pcount = $pcount + 1;
     }
-    if (($type eq "AIFF") || ($type eq "MOD ") || ($type eq "OGGV")
-		|| ($type eq "WAVE") || ($type eq "MP3 "))
+    if (isaudio($type))
     {   $sound_numbering[$chunk_number_array[$x]] = $x;
         $scount++;
     }
