@@ -3,77 +3,77 @@
 System_file;
 
 Constant PUNYINFORM_MAJOR_VERSION = 3;
-Constant PUNYINFORM_MINOR_VERSION = 1;
+Constant PUNYINFORM_MINOR_VERSION = 5;
 
-#Ifndef VN_1635;
-Message fatalerror "*** The PunyInform library needs Inform v6.35 or later to work ***";
+#Ifndef VN_1636;
+Message fatalerror "*** The PunyInform library needs Inform v6.36 or later to work ***";
 #Endif; ! VN_
 
 #IfnDef CUSTOM_ABBREVIATIONS;
-Abbreviate "You ";
-Abbreviate "thing";
-Abbreviate "n't ";
-Abbreviate "This game is now in its ";
-Abbreviate " you ";
-Abbreviate " descriptions of locations ";
-Abbreviate "There is no";
-Abbreviate " to ";
-Abbreviate " that";
-Abbreviate "lready ";
-Abbreviate "which ";
-Abbreviate "ed.";
-Abbreviate "ing ";
-Abbreviate "nderstand";
-Abbreviate " the ";
-Abbreviate "here";
-Abbreviate " no";
-Abbreviate "That";
-Abbreviate "Inform";
-Abbreviate "You're";
-Abbreviate ".^";
-Abbreviate "some";
-Abbreviate "have";
-Abbreviate "~ mode, ";
-Abbreviate "would ";
-Abbreviate "tion";
+Abbreviate "are referring to.";
 Abbreviate " fixed in place.";
-Abbreviate "nexpect";
-Abbreviate "are";
-Abbreviate "can ";
-Abbreviate ", ";
-Abbreviate "yourself";
+Abbreviate "n't understand";
+Abbreviate "(first taking ";
+Abbreviate " doesn't seem";
+Abbreviate " seems to be";
+Abbreviate "But you are";
+Abbreviate "unexpected.";
+Abbreviate "description";
+Abbreviate "You can't ";
+Abbreviate " nothing ";
+Abbreviate "something";
+Abbreviate " carrying";
+Abbreviate "There is";
+Abbreviate "multiple";
+Abbreviate "LookMode";
+Abbreviate " already";
+Abbreviate "irection";
+Abbreviate " (which ";
+Abbreviate "contain";
+Abbreviate "/when_o";
+Abbreviate "You are";
 Abbreviate "before";
-Abbreviate "I do";
-Abbreviate "see";
-Abbreviate "always gives ";
-Abbreviate " ***";
-Abbreviate "e.";
-Abbreviate " has better ";
-Abbreviate "Cannot put ";
-Abbreviate "is clos";
-Abbreviate " number";
-Abbreviate "achieve";
-Abbreviate "But";
+Abbreviate "_array";
+Abbreviate "essage";
+Abbreviate "number";
+Abbreviate "would ";
+Abbreviate "closed";
+Abbreviate "You're";
+Abbreviate "object";
+Abbreviate "That's";
+Abbreviate " the ";
+Abbreviate " you ";
+Abbreviate "which";
+Abbreviate "thing";
+Abbreviate "witch";
+Abbreviate " that";
+Abbreviate "parse";
+Abbreviate "react";
+Abbreviate "able";
+Abbreviate "can ";
+Abbreviate "name";
+Abbreviate "n't ";
+Abbreviate " to ";
+Abbreviate "_to/";
+Abbreviate "see ";
+Abbreviate "ing ";
+Abbreviate "here";
+Abbreviate "You ";
+Abbreviate "have";
+Abbreviate "self";
+Abbreviate "ed.";
+Abbreviate " no";
+Abbreviate "est";
+Abbreviate "ut ";
 Abbreviate " in";
-Abbreviate " of";
-Abbreviate "(first tak";
-Abbreviate "The ";
-Abbreviate " (";
-Abbreviate "'s";
-Abbreviate " by this.";
-Abbreviate " happens.";
-Abbreviate "lease ";
-Abbreviate "switch";
-Abbreviate " empty";
-Abbreviate "moment";
-Abbreviate "pecial";
-Abbreviate "rest";
-Abbreviate " error: ";
-Abbreviate "(even if";
-Abbreviate " itself.";
-Abbreviate " an";
-Abbreviate " on";
-Abbreviate "hat";
+Abbreviate "ope";
+Abbreviate "ent";
+Abbreviate "eve";
+Abbreviate "Th";
+Abbreviate "I ";
+Abbreviate ", ";
+Abbreviate "In";
+Abbreviate "e.";
 #EndIf;
 
 #IfDef STATUSLINE_TIME;
@@ -86,6 +86,41 @@ Constant Grammar__Version = 2;
 Constant INDIV_PROP_START 64;
 Constant NULL         = $ffff;
 
+! Z-code header constants
+Constant HDR_ZCODEVERSION  $00;     ! byte
+Constant HDR_TERPFLAGS     $01;     ! byte
+Constant HDR_GAMERELEASE   $02;     ! word
+Constant HDR_HIGHMEMORY    $04;     ! word
+Constant HDR_INITIALPC     $06;     ! word
+Constant HDR_DICTIONARY    $08;     ! word
+Constant HDR_OBJECTS       $0A;     ! word
+Constant HDR_GLOBALS       $0C;     ! word
+Constant HDR_STATICMEMORY  $0E;     ! word
+Constant HDR_GAMEFLAGS     $10;     ! word
+Constant HDR_GAMESERIAL    $12;     ! six ASCII characters
+Constant HDR_ABBREVIATIONS $18;     ! word
+Constant HDR_FILELENGTH    $1A;     ! word
+Constant HDR_CHECKSUM      $1C;     ! word
+Constant HDR_TERPNUMBER    $1E;     ! byte
+Constant HDR_TERPVERSION   $1F;     ! byte
+Constant HDR_SCREENHLINES  $20;     ! byte
+Constant HDR_SCREENWCHARS  $21;     ! byte
+Constant HDR_SCREENWUNITS  $22;     ! word
+Constant HDR_SCREENHUNITS  $24;     ! word
+Constant HDR_FONTWUNITS    $26;     ! byte
+Constant HDR_FONTHUNITS    $27;     ! byte
+Constant HDR_ROUTINEOFFSET $28;     ! word
+Constant HDR_STRINGOFFSET  $2A;     ! word
+Constant HDR_BGCOLOUR      $2C;     ! byte
+Constant HDR_FGCOLOUR      $2D;     ! byte
+Constant HDR_TERMCHARS     $2E;     ! word
+Constant HDR_PIXELSTO3     $30;     ! word
+Constant HDR_TERPSTANDARD  $32;     ! two bytes
+Constant HDR_ALPHABET      $34;     ! word
+Constant HDR_EXTENSION     $36;     ! word
+Constant HDR_UNUSED        $38;     ! two words
+Constant HDR_INFORMVERSION $3C;     ! four ASCII characters
+
 Constant PHASE1 = 1; ! just check if pattern matches (no side effects such as which? questions or indirect actions)
 Constant PHASE2 = 2; ! parse the given pattern with side effects
 Constant PHASE2_SUCCESS         = 0;
@@ -107,6 +142,8 @@ Constant comma_word   = 'comma,';  ! An "untypeable word" used to substitute
 Constant SOMETHING_STR = "something";
 Constant SOMEONE_STR = "someone";
 Constant SOMEDIRECTION_STR = "(some direction)";
+Constant IS_STR = "is ";
+Constant ARE_STR = "are ";
 
 #Ifndef DEFAULT_CAPACITY;
 Constant DEFAULT_CAPACITY = 100;
@@ -119,6 +156,7 @@ Message warning "*** Skipping Undo - not supported in v3 ***";
 Constant OPTIONAL_PROVIDE_UNDO_FINAL;
 #Endif;
 #Endif;
+#Ifdef DEATH_MENTION_UNDO; #Endif;
 
 
 #Ifndef OPTIONAL_NO_DARKNESS;
@@ -277,13 +315,6 @@ Array LanguageNumbers static table
 #EndIf;
 
 
-! Header constants
-Constant HDR_DICTIONARY      = $8;   ! Actual address
-Constant HDR_STATIC_MEM      = $e;   ! Actual address
-Constant HDR_SCREENHCHARS    = $20;
-Constant HDR_SCREENWCHARS    = $21;
-Constant HDR_FLAGS2          = $10;  ! transcript, undo etc.
-
 Constant TT_OBJECT           = 1;    ! one or more words referring to an object
                                      ! it is one of NOUN_TOKEN etc. below
 Constant TT_PREPOSITION      = 2;    ! e.g. 'into'
@@ -334,8 +365,8 @@ Constant SCORE__TX = " Score: ";
 Constant TIME__TX = " Time: ";
 #EndIf;
 
-#Default Story        0;
-#Default Headline     0;
+!#Default Story        0;
+!#Default Headline     0;
 
 Default MAX_SCORE           0;
 Default AMUSING_PROVIDED    1;
@@ -382,6 +413,7 @@ Global wn;               ! word number within parse array
 Global pattern_pointer;  ! token within current pattern
 Global num_words;        ! number of words typed
 Global action;           ! the current action
+Global action_to_be;     ! the action being considered
 Global action_reverse;   ! if parameters are in reversed order
 Global meta;             ! if the verb has the meta attribute or not
 Global update_moved;     ! if _NoteObjectAcquisitions should update moved
@@ -394,6 +426,7 @@ Global special_word;     ! Dictionary address for "special" token
 Global special_number;   ! Number typed special orders, e.g "john, 34"
 Global parsed_number;    ! For user-supplied parsing routines
 Global noun_filter;      ! For noun filtering by user routines
+Global object_token_type;! Remember what kind of object token type is worked on
 Global noun;
 Global second;
 Global selected_direction;
@@ -406,6 +439,7 @@ Global parser_check_multiple;    ! parser should check if multiheld/multiinside
 Global parser_unknown_noun_found;! parser should report unknown word
 Global parser_all_found;! parser encountered 'all'
 Global parser_all_except_object; ! used to filter all but/except patterns
+GLobal parser_one; ! Used to pass extra information from user code to parser
 Global usual_grammar_after; ! needed for grammar property parsing
 Global deadflag;
 Global scope_modified;   ! true if daemons, each_turn etc has invalidated scope
@@ -420,6 +454,7 @@ Global themobj = 0;       ! The object which is currently "them"
 Global top_object;
 Global newline_flag;     ! Used by Look
 Global also_flag;        ! Used by Look
+Global inventory_style = 1;
 Global inventory_stage;
 Global phase2_necessary;
 Global receive_action;
@@ -465,7 +500,8 @@ Default MAX_INPUT_CHARS       78;
 Default MAX_INPUT_WORDS       20;
 Default MAX_SCOPE             32;
 
-Global scope_pov;        ! Whose POV the scope is from (usually the player)
+Global cached_scope_pov;        ! Whose POV the cached scope is from (usually the player)
+Global cached_scope_routine;    ! Which scope routine the cached scope is for
 Array scope-->MAX_SCOPE; ! objects visible from the current POV
 Array scope_copy-->MAX_SCOPE; ! Used to hold a copy of a scope list, for iteration
 
@@ -484,6 +520,12 @@ Constant MAX_FLOATING_OBJECTS  32;            ! Max number of objects that have
 #Endif; ! MAX_FLOATING_OBJECTS
 Array floating_objects --> MAX_FLOATING_OBJECTS + 1;
 
+Constant WORKFLAG_BIT  $0001;       ! At top level (only), only list objects
+                                    ! which have the "workflag" attribute
+Constant ISARE_BIT     $0002;       ! Print " is" or " are" before list
+Constant NEWLINE_BIT   $0004;       ! Print newline after each entry
+Global pc_indent = 0;				! 0 means PrintContents is not running
+
 Array which_object-->MAX_WHICH_OBJECTS;       ! options for "which book?"
 Array which_level-->MAX_WHICH_OBJECTS;        ! type of which_object
 Array multiple_objects-->MAX_MULTIPLE_OBJECTS;! holds nouns when multi* used
@@ -540,9 +582,12 @@ Object Directions
 #IfNot;
 		parse_name [_parse _i _w;
 #EndIf;
-			_parse = parse+4*wn-2;
-			_w = _parse-->0;
 #IfV5;
+!			_parse = parse+4*wn-2;
+			@log_shift wn 2 -> _parse; ! Multiply by 4
+			_parse = parse + _parse - 2;
+
+			_w = _parse-->0;
 			_arr = _direction_dict_words;
 			if(normal_directions_enabled) {
 				@scan_table _w _arr (DIRECTION_COUNT * 2) -> _i ?success;
@@ -577,6 +622,10 @@ Object Directions
 			return 1;
 #IfNot;
 			! This is V3
+
+			_parse = parse+4*wn-2;
+			_w = _parse-->0;
+
 			if(normal_directions_enabled) {
 #Ifndef OPTIONAL_SHIP_DIRECTIONS;
 				@je _w 'out' ?match_out;
@@ -614,8 +663,10 @@ Object Directions
 			}
 
 #Endif; ! OPTIONAL_SHIP_DIRECTIONS
-			jump fail;
-
+      		! No direction was matched
+!			selected_direction_index = 0;
+!			selected_direction = 0;
+			return 0;
 
 .match_out; @inc _i;
 .match_in; @inc _i;
@@ -637,11 +688,6 @@ Object Directions
 .match2;
 			selected_direction = direction_properties_array -> selected_direction_index;
 			return 1;
-.fail;
-      		! No direction was matched
-!			selected_direction_index = 0;
-!			selected_direction = 0;
-			return 0;
 #EndIf;
 		],
 has scenery
