@@ -48,8 +48,14 @@ Verb 'drop' 'discard' 'throw'
 Verb 'eat'
     * held                                      -> Eat;
 
+#IfDef OPTIONAL_EXTENDED_VERBSET;
+Verb 'enter'
+	*                                           -> GoIn
+	* noun                                      -> Enter;
+#IfNot;
 Verb 'enter'
 	* noun                                      -> Enter;
+#Endif;
 
 Verb 'examine' 'x//'
 	* noun -> Examine;
@@ -86,8 +92,14 @@ Verb 'go'
 Verb 'insert'
     * multiexcept 'in'/'into' noun              -> Insert;
 
+#Ifdef OPTIONAL_FLEXIBLE_INVENTORY;
+Verb 'inventory' 'i//'
+	* -> Inv
+	* 'tall'/'wide' -> Inv;
+#Ifnot;
 Verb 'inventory' 'i//'
 	* -> Inv;
+#Endif;
 
 Verb 'jump'
 	*                                           -> Jump
@@ -104,9 +116,9 @@ Verb 'lock'
 	* noun 'with' held                          -> Lock;
 
 Verb 'look' 'l//'
-	*                                           -> Look
 	* 'at' noun                                 -> Examine
-	* 'in'/'inside' noun						-> Search;
+	*                                           -> Look
+	* 'in'/'inside'/'on' noun                   -> Search;
 
 Verb 'open' 'uncover' 'unwrap'
 	* noun                                      -> Open
@@ -159,8 +171,8 @@ Verb 'show' 'display' 'present'
 	* held 'to' creature                        -> Show;
 
 Verb 'sit' 'lie'
-	* 'on' 'top' 'of' noun                      -> Enter
-	* 'on'/'in'/'inside' noun                   -> Enter;
+	* 'on'/'in'/'inside' noun                   -> Enter
+	* 'on' 'top' 'of' noun                      -> Enter;
 
 Verb 'smell' 'sniff'
 	*                                           -> Smell
@@ -180,7 +192,7 @@ Verb 'switch'
 
 Verb 'take' 'carry' 'hold'
 	* multi                                     -> Take
-	* 'off' worn                                -> Disrobe
+	* 'off' noun                                -> Disrobe
 	* multiinside 'from'/'off' noun             -> Remove
 	* 'inventory'                               -> Inv;
 
@@ -192,7 +204,7 @@ Verb 'tie' 'attach' 'fasten' 'fix'
 	* noun                                      -> Tie
 	* noun 'to' noun                            -> Tie;
 
-Verb 'touch'
+Verb 'touch' 'feel' 'fondle' 'grope'
 	* noun                                      -> Touch;
 
 Verb 'turn' 'rotate' 'screw' 'twist' 'unscrew'
@@ -346,7 +358,12 @@ Verb 'wear'
 
 [ ExitSub _p;
 	_p = parent(player);
-	if(parent(_p) == 0) { PrintMsg(MSG_EXIT_ALREADY); rtrue; }
+	if(parent(_p) == 0) {
+		! player not inside, standing in the room
+		if(_p.out_to && noun == 0) <<Go FAKE_OUT_OBJ>>;
+		PrintMsg(MSG_EXIT_ALREADY); 
+		rtrue; 
+	}
 	if(noun == 0) <<Exit _p>>;
 	if(player notin noun) {
 		if(IndirectlyContains(noun, player)) { PrintMsg(MSG_EXIT_FIRST_LEAVE, parent(player)); rtrue; }
@@ -500,7 +517,14 @@ Array _InsertMessages -->
 	_MoveNounToSecond(_InsertMessages);
 ];
 
+#Ifdef OPTIONAL_FLEXIBLE_INVENTORY;
+[ InvSub _mode;
+	_mode = WordValue(num_words);
+	if(_mode == 'tall') inventory_style = 0;
+	if(_mode == 'wide') inventory_style = 1;
+#Ifnot;
 [ InvSub;
+#Endif;
     if(PrintMsg(MSG_INVENTORY_DEFAULT) == false) {
 		PrintMsg(MSG_INVENTORY_EMPTY);
 	}
@@ -609,14 +633,15 @@ Array _PutOnMessages -->
 	PrintMsg(MSG_RUB_DEFAULT);
 ];
 
-[ SearchSub _f _i;
+[ SearchSub _i _plural;
 #Ifndef OPTIONAL_NO_DARKNESS;
 	if(location == thedark) { PrintMsg(MSG_SEARCH_DARK); rtrue; }
 #Endif;
 	if (ObjectIsUntouchable(noun)) return;
-	objectloop(_i in noun) if(_i hasnt concealed && _i hasnt scenery) _f++;
+	_plural = PrintContents(1, noun);
+
 	if(noun has supporter) {
-		if(_f == 0)
+		if(_plural == 0)
 			PrintMsg(MSG_SEARCH_NOTHING_ON);
 		else
 			PrintMsg(MSG_SEARCH_ON_IT_ISARE);
@@ -628,7 +653,7 @@ Array _PutOnMessages -->
 	}
 	if(AfterRoutines()) rtrue;
 	_i = MSG_SEARCH_EMPTY;
-	if(_f)
+	if(_plural)
 		_i = MSG_SEARCH_IN_IT_ISARE;
 	PrintMsg(_i);
 ];
@@ -1019,10 +1044,8 @@ Verb meta 'restore'
 Verb meta 'save'
 	*                                           -> Save;
 
-#Ifndef NO_SCORE;
 Verb meta 'score'
     *                                           -> Score;
-#Endif;
 
 Verb meta 'superbrief' 'short'
 	*                                           -> LookModeShort;
@@ -1162,7 +1185,11 @@ Verb meta 'quit' 'q//'
 #EndIf;
 ];
 
-#Ifndef NO_SCORE;
+#Ifdef NO_SCORE;
+[ ScoreSub;
+	PrintMsg(MSG_SCORE_DEFAULT);
+];
+#Ifnot;
 [ ScoreSub;
 	PrintMsg(MSG_SCORE_DEFAULT);
 	PrintRank();
@@ -1171,23 +1198,33 @@ Verb meta 'quit' 'q//'
 
 [ Banner _i;
 	new_line;
-	if(Story ~= 0) {
-#IfV5;
+#IfDef Story;
+	#IfV5;
 		style bold;
-#EndIf;
+	#EndIf;
 		print (string) Story;
-#IfV5;
+	#IfV5;
 		style roman;
+	#EndIf;
+	#IfDef Headline;
+		print (string) Headline;
+	#EndIf;
 #EndIf;
-		if(Headline ~= 0) print (string) Headline;
-    }
 	print "Release ", (0-->1) & $03ff, " / Serial number ";
 	for (_i = 18:_i < 24: _i++) print (char) 0->_i;
 	print " / Inform v";
 	inversion;
 	print " PunyInform v", PUNYINFORM_MAJOR_VERSION, (char) '.', PUNYINFORM_MINOR_VERSION;
+	_i = 0;
+#IfDef STRICT_MODE;
+	#IfV5;
+    print " S";
+    _i = 1;
+    #EndIf;
+#EndIf;
 #IfDef DEBUG;
-	print " D";
+	if(_i == 0) print " ";
+	print "D";
 #EndIf;
 	@new_line;
 ];
@@ -1290,21 +1327,21 @@ Verb meta 'verify'
 #Endif; ! NO_PLACES
 
 [ ScriptOnSub;
-	transcript_mode = ((HDR_FLAGS2-->0) & 1);
+	transcript_mode = ((HDR_GAMEFLAGS-->0) & 1);
 	if (transcript_mode) "Transcripting is already on.";
 	@output_stream 2;
-	if (((HDR_FLAGS2-->0) & 1) == 0) "Attempt to begin transcript failed.";
+	if (((HDR_GAMEFLAGS-->0) & 1) == 0) "Attempt to begin transcript failed.";
 	print "Start of a transcript of^";
 	Banner();
 	transcript_mode = true;
 ];
 
 [ ScriptOffSub;
-	transcript_mode = ((HDR_FLAGS2-->0) & 1);
+	transcript_mode = ((HDR_GAMEFLAGS-->0) & 1);
 	if (transcript_mode == false) "Transcripting is already off.";
 	print "^End of transcript.^";
 	@output_stream -2;
-	if ((HDR_FLAGS2-->0) & 1) "Attempt to end transcript failed.";
+	if ((HDR_GAMEFLAGS-->0) & 1) "Attempt to end transcript failed.";
 	transcript_mode = false;
 ];
 
@@ -1401,7 +1438,6 @@ Global scope_cnt;
 [ _ScopeSubHelper p_obj;
 	print scope_cnt++,": ", (a) p_obj, " (", p_obj, ")";
 	if(ObjectIsUntouchable(p_obj, true)) print " [untouchable]";
-	if(ObjectIsInvisible(p_obj, true)) print " [invisible]";
 	@new_line;
 ];
 
@@ -1499,7 +1535,8 @@ Global scope_cnt;
 ! HELP ROUTINES
 ! ---------------------
 
-[ _ListObjsMsg;
+[ _ListObjsMsg p_parent;
+	p_parent = 0; ! Avoid warning
 	print "^You can ";
 	if(also_flag) print "also ";
 	print "see ";
@@ -1514,7 +1551,7 @@ Global scope_cnt;
 	print "see ";
 ];
 
-[ Look _obj _top_ceil _ceil _initial_found _describe_room
+[ Look _obj _top_ceil _ceil _describe_room
 	_you_can_see_1 _you_can_see_2 _desc_prop _last_level _action;
 	PrintMsg(MSG_LOOK_BEFORE_ROOMNAME);
 	if((lookmode == 1 && location hasnt visited) || lookmode == 2) _describe_room = true;
@@ -1584,7 +1621,6 @@ Global scope_cnt;
 			give _obj workflag;
 			if(_obj.&describe) {
 				if(PrintOrRun(_obj, describe, 0)) {
-					_initial_found = true;
 					give _obj ~workflag;
 					also_flag = true;
 					continue;
@@ -1606,7 +1642,6 @@ Global scope_cnt;
 				_desc_prop = initial;
 			}
 			if(_obj.&_desc_prop && (_obj hasnt moved || _desc_prop == when_off)) { ! Note: when_closed in an alias of when_off
-				_initial_found = true;
 				give _obj ~workflag;
 				@new_line;
 				PrintOrRun(_obj, _desc_prop);
@@ -1746,7 +1781,7 @@ Global scope_cnt;
 	rfalse;
 ];
 
-[ GoDir p_property _new_location _door_to _vehicle _vehicle_mode _saved_location;
+[ GoDir p_property _new_location _old_location _door_to _vehicle _vehicle_mode _saved_location;
 	if(parent(player) ~= real_location) {
 		! special rule when in enterable (veichles)
 		! before routine for the object is called with Go dir, and returns
@@ -1812,6 +1847,11 @@ Global scope_cnt;
 	! If _new_location is 0, we tell the player they can't go there and exit
 	if(_new_location == 0) {
 		if(real_location provides cant_go) {
+#IfDef DEBUG;
+#IfV3;
+			if(debug_flag & 1) print "(", (name) real_location, ").cant_go()^";
+#EndIf;
+#EndIf;
 			PrintOrRun(real_location, cant_go);
 			rtrue;
 		}
@@ -1819,18 +1859,32 @@ Global scope_cnt;
 		rtrue;
 	}
 
+#IfDef DEBUG;
+#IfV3;
+	if(debug_flag & 1) print "(", (name) _new_location, ").before()^";
+#EndIf;
+#EndIf;
 	action = ##Going;
-	if (RunRoutines(_new_location, before) ~= 0) { action = ##Go; rtrue; }
+	if(RunRoutines(_new_location, before)) { action = ##Go; rtrue; }
 	action = ##Go;
 
 	if(_vehicle_mode == 1) {
 		move _vehicle to _new_location;
 		_new_location = _vehicle;
 	}
+
+	_old_location = location;
+
 	PlayerTo(_new_location, true);
 	if(deadflag ~= GS_PLAYING) rtrue;
-	if (AfterRoutines()) rtrue;
-	if (keep_silent) rtrue;
+
+	action = ##Going;
+	if(RunRoutines(_old_location, after)) { action = ##Go; rtrue; }
+	action = ##Go;
+
+
+	if(AfterRoutines()) rtrue;
+	if(keep_silent) rtrue;
 	Look();
 ];
 
