@@ -126,7 +126,7 @@ Constant MSG_PARSER_NOT_MULTIPLE_DIRS "You can't use multiple directions.";
 Constant MSG_PARSER_BAD_NUMBER "I didn't understand that number.";
 #EndIf;
 #Ifndef MSG_PARSER_NO_INPUT;
-Constant MSG_PARSER_NO_INPUT "Come again?^";
+Constant MSG_PARSER_NO_INPUT "Come again?";
 #EndIf;
 #Ifndef MSG_PARSER_UNKNOWN_SENTENCE;
 Constant MSG_PARSER_UNKNOWN_SENTENCE "I don't understand that sentence.";
@@ -1014,7 +1014,13 @@ MSG_RUB_DEFAULT, MSG_SQUEEZE_DEFAULT:
 	MSG_RESTART_RESTORE_OR_QUIT:
 		print "^Would you like to RESTART, RESTORE";
 #Ifdef OPTIONAL_PROVIDE_UNDO_FINAL;
-		print ", UNDO the last move";
+	#Ifdef DEATH_MENTION_UNDO;
+			if(((HDR_GAMEFLAGS->1) & 16) ~= 0)
+				print ", UNDO the last move";
+	#Ifnot;
+			if(((HDR_GAMEFLAGS->1) & 16) ~= 0 && deadflag ~= GS_WIN)
+				print ", UNDO the last move";
+	#Endif;
 #Endif;
 #IfDef OPTIONAL_FULL_SCORE;
 		print ", give the FULL score for that game";
@@ -1120,7 +1126,7 @@ default:
 	if (obj has pluralname) { print "them"; rtrue; }
 	if (obj has animate) {
 		if (obj has female) { print "her"; rtrue; }
-		else if (obj hasnt neuter) { print "him"; rtrue; }
+		if (obj hasnt neuter) { print "him"; rtrue; }
 	}
 	print "it";
 ];
@@ -1138,7 +1144,7 @@ default:
 	if (obj has pluralname)		{ print "They're"; return; }
 	if (obj has animate) {
 		if (obj has female)		{ print "She's"; return; }
-		else if (obj hasnt neuter) { print "He's"; return; }
+		if (obj hasnt neuter) { print "He's"; return; }
 	}
 	print "That's";
 ];
