@@ -32,7 +32,7 @@ System_file;
 	return _ret;
 ];
 
-[ _FixIncompleteSentenceOrComplain p_pattern _token _type _data _noun _prep _second _num_preps _num_nouns;
+[ _FixIncompleteSentenceOrComplain p_pattern _token _type _noun _prep _second _num_nouns;
 	! Called because sentence shorter than the pattern
 	! Available data: wn, parse and p_pattern_token (last matched token)
 	!
@@ -78,6 +78,13 @@ System_file;
 #EndIf;
 
 	! write an error message and return false
+	_PrintPatternSyntax(p_pattern, _noun);
+	rfalse;
+];
+
+[ _PrintPatternSyntax p_pattern p_noun _num_preps _token _type _data;
+	! write what pattern we expected, something like:
+	! "I think you wanted to say "put all in something". Please try again."
 	PrintMsg(MSG_PARSER_BAD_PATTERN_PREFIX);
 	print (verbname) verb_word;
 	for(_token = p_pattern + 3: _token->0 ~= TT_END: _token = _token + 3) {
@@ -91,7 +98,7 @@ System_file;
 			++_num_preps;
 		} else {
 			@print_char ' ';
-			if(_noun == 0) {
+			if(p_noun == 0) {
 				if(_type == TT_ROUTINE_FILTER && _data == ADirection) {
 					print (string) SOMEDIRECTION_STR;
 				} else if(second == 0) {
@@ -103,7 +110,7 @@ System_file;
 				} else print (name) second;
 			} else {
 				if(noun ~= 0) {
-					_noun = 0; ! avoid repeat (and we don't need _noun anymore)
+					p_noun = 0; ! avoid repeat (we don't need p_noun anymore)
 					if(parser_all_found) print "all"; else print (name) noun;
 				} else if(_type == TT_OBJECT && _token->2 == CREATURE_OBJECT) {
 					print (string) SOMEONE_STR;
@@ -114,7 +121,6 @@ System_file;
 		}
 	}
 	PrintMsg(MSG_PARSER_BAD_PATTERN_SUFFIX);
-	rfalse;
 ];
 
 [ _AskWhichNoun p_num_matching_nouns _i;
@@ -810,7 +816,7 @@ System_file;
 [ _IsSentenceDivider p_parse_pointer;
 	! check if current parse block, indicated by p_parse_pointer,
 	! is a period or other sentence divider
-	if(p_parse_pointer --> 0 == './/' or ',//' or 'and' or 'then') rtrue;
+	if(p_parse_pointer --> 0 == './/' or ',//' or AND_WORD or THEN1__WD) rtrue;
 	rfalse;
 ];
 
@@ -1466,12 +1472,7 @@ Array guess_object-->5;
 #Endif;
 				continue; ! keep parsing
 			}
-
-			! write error messages if PHASE2 as needed
-			if(pattern_pointer->0 == TOKEN_LAST_PREP or TOKEN_SINGLE_PREP) {
-				! bad preposition
-				if(p_phase == PHASE2) PrintMsg(MSG_PARSER_UNKNOWN_SENTENCE);
-			} else if(parser_unknown_noun_found ~= 0) {
+			if(parser_unknown_noun_found ~= 0) {
 				if(p_phase == PHASE2) {
 					_word = parser_unknown_noun_found --> 0;
 					if(scope_routine ~= 0) {
@@ -1585,7 +1586,7 @@ Array guess_object-->5;
 	!print "he ", himobj, " she ", herobj, " it ", itobj, "^";
 ];
 
-[ _ParseAndPerformAction _word_data _verb_grammar _i _j _pattern _noun _score _best_score _best_pattern _best_pattern_pointer _best_phase2 _action;
+[ _ParseAndPerformAction _word_data _verb_grammar _i _j _pattern _noun _score _best_score _best_pattern _best_pattern_pointer _best_phase2 _action _verb_offset;
 	! returns
 	! 1/true: if error was found
 	! -n: if <n> words were used to find a match,
@@ -1611,6 +1612,7 @@ Array guess_object-->5;
 	consult_from = 0;
 	consult_words = 0;
 	usual_grammar_after = 0;
+	_verb_offset = 0;
 	scope_routine = 0;
 	noun_filter = 0;
 	object_token_type = -1;
@@ -1690,11 +1692,17 @@ Array guess_object-->5;
 		verb_wordnum = wn;
 		verb_word = (parse - 2) --> (2 * verb_wordnum);
 		if(actor provides grammar) {
+			parser_one = 0;
 			_i = actor.grammar();
+			! 0 = carry on as usual
+			! 1 = grammar parsed it completely
+			! verb = use this verb's grammar instead
+			! -verb = try using this verb's grammar first, then the original
 			if((_i ~= 0 or 1) &&
+				(parser_one ~= 0 ||
 				(UnsignedCompare(_i, dict_start) < 0 ||
 				UnsignedCompare(_i, dict_end) >= 0 ||
-				(_i - dict_start) % dict_entry_size ~= 0)) {
+				(_i - dict_start) % dict_entry_size ~= 0))) {
 				! returned -'verb'
 				usual_grammar_after = verb_word;
 				_i = -_i;
@@ -1705,8 +1713,12 @@ Array guess_object-->5;
 			}
 			if(_i ~= 0) {
 				! _i == 'verb', so use its grammar instead
-				verb_wordnum = wn;
 				verb_word = _i;
+				! decrease verb_wordnum to try the new grammar rule,
+				! but keep track of the offset so we can
+				! restore in case the rule fails.
+				_verb_offset = 1;
+				verb_wordnum = verb_wordnum - _verb_offset;
 				jump reparse2;
 			}
 		}
@@ -1722,7 +1734,7 @@ Array guess_object-->5;
 !   print "Word 2: ", (parse + 6)-->0, "^";
 !   print "Word 3: ", (parse + 10)-->0, "^";
 	_i = 255 - (_word_data->1); ! was _verb_num
-	_verb_grammar = (HDR_STATIC_MEM-->0)-->_i;
+	_verb_grammar = (HDR_STATICMEMORY-->0)-->_i;
 
 #IfDef DEBUG_PARSEANDPERFORM;
 	print "Verb#: ",_i,", meta ",meta,".^";
@@ -1749,7 +1761,7 @@ Array guess_object-->5;
 		_score = _ParsePattern(_pattern, PHASE1);
 
 #IfDef DEBUG_PARSEANDPERFORM;
-		print "### PHASE 1: result ", _score, " phase2 ", phase2_necessary, "^";
+		print "### PHASE 1: result ", _score, " phase2 ", phase2_necessary, " wn ", wn, "^";
 #EndIf;
 		! note that _ParsePattern will never return -1 in PHASE1
 		if(_score == 0) {
@@ -1798,6 +1810,13 @@ Array guess_object-->5;
 #EndIf;
 		jump parse_success;
 	}
+	if(_verb_offset ~= 0) {
+		! restore the normal verb_wordnum after trying a
+		! rule suggested by the NPC's grammar property
+		! which, at this point, has failed.
+		verb_wordnum = verb_wordnum + _verb_offset;
+		_verb_offset = 0;
+	}
 	if(usual_grammar_after ~= 0) {
 		! we have parsed a grammar given by the grammar property
 		! which failed, so we should reparse using the original
@@ -1834,11 +1853,13 @@ Array guess_object-->5;
 				}
 			} else {
 				! we didn't match the pattern at all
-				if(parser_unknown_noun_found == 0 &&
-					((_best_pattern_pointer-> 0) & $0f) == TT_END) {
-					PrintMsg(MSG_PARSER_PARTIAL_MATCH, wn - 1);
+				_i = ((_best_pattern - 1 + wn*3 )-> 0) & $0f;
+				if(_i == TT_PREPOSITION or TT_OBJECT) {
+					! missing preposition
+					_PrintPatternSyntax(_best_pattern -1);
 				} else {
-					PrintMsg(MSG_PARSER_NOSUCHTHING);
+					! some other problem
+					PrintMsg(MSG_PARSER_PARTIAL_MATCH, wn - 1);
 				}
 			}
 		}
