@@ -1,50 +1,18 @@
 ! PunyInform: A small stdlib and parser for interactive fiction
 ! suitable for old-school computers such as the Commodore 64.
 ! Designed to be similar, but not identical, to the Inform 6 library.
-!
-! Reference documentation
-! DM: http://www.inform-fiction.org/manual/html/dm4index.html
+
+! Most public routines, as described in DM4, are available for a game
+! developer. See the manual in the documentation folder for a full list
+! of these routines, as well as additional useful routines that are
+! PunyInform only.
+
+! Reference documentation:
+! DM4: http://www.inform-fiction.org/manual/html/dm4index.html
 ! Tech: https://www.inform-fiction.org/source/tm/TechMan.txt
 ! Z-machine: https://www.inform-fiction.org/zmachine/standards/z1point1
-!
-! Public routines (described in DM, available for a game developer)
-! LIBRARY ROUTINES:
-! - CommonAncestor
-! - DrawStatusLine
-! - IndirectlyContains
-! - InScope
-! - LoopOverScope
-! - NextWord
-! - NextWordStopped
-! - NumberWord
-! - NumberWords
-! - ObjectIsUntouchable
-! - PlayerTo
-! - ParseToken
-! - PlaceInScope
-! - PronounNotice
-! - ScopeWithin
-! - SetTime
-! - TestScope
-! - TryNumber
-! - WordAddress
-! - WordLength
-! - WordValue
-! - YesOrNo
-! OTHERS:
-! - PrintOrRun
-! - RunRoutines
-! PRINT UTILITIES:
-! - CTheyreorThats
-! - ItorThem
-! - IsOrAre
-! PUNYINFORM ONLY:
-! - OnOff
-! - ObjectIsInvisible
-! - PrintMsg
-! - RunTimeError
 
-! comment/uncomment to restrict default debug messages behaviour
+! Comment/uncomment to restrict default debug messages behaviour
 ! (all can be overridden by adding them in the game source)
 !Constant DEBUG_SCOPE;
 !Constant DEBUG_CHECKNOUN;
@@ -60,7 +28,6 @@ Include "messages.h";
 
 ! ######################### Include utility files
 
-Include "scope.h";
 Include "grammar.h";
 
 ! ######################### Helper routines
@@ -112,13 +79,6 @@ Include "grammar.h";
     return 0;
 ];
 
-[ SetTime p_time p_step;
-	the_time = p_time;
-	time_rate = p_step;
-	time_step = 0;
-	if(p_step < 0) time_step = -p_step;
-];
-
 #IfV5;
 
 Array cursor_pos --> 2;
@@ -169,16 +129,11 @@ Array TenSpaces -> "          ";
 	@get_cursor cursor_pos;
 	_current_col = cursor_pos --> 1;
 
-	if(_current_col > p_col || cursor_pos --> 0 > 1) {
+	if(_current_col > p_col || cursor_pos --> 0 > 1)
 		_MoveCursor(1, p_col);
-	} else {
-		p_col = p_col - _current_col;
-		while(p_col > 10) {
-			@print_table TenSpaces 10;
-			p_col = p_col - 10;
-		}
-		@print_table TenSpaces p_col;
-	}
+	else
+		FastSpaces(p_col - _current_col);
+
 	if(p_string)
 		print (string) p_string;
 ];
@@ -361,8 +316,12 @@ else
 #IfDef SACK_OBJECT;
 [ _RoomInSack _obj _ks;
     if (SACK_OBJECT in player) {
-        for (_obj=youngest(player) : _obj : _obj=elder(_obj))
+        for (_obj=youngest(player) : _obj : _obj=elder(_obj)) {
+#Ifdef OPTIONAL_NO_DARKNESS;
+            if (_obj ~= SACK_OBJECT && _obj hasnt worn) {
+#Ifnot;
             if (_obj ~= SACK_OBJECT && _obj hasnt worn or light) {
+#Endif;
                 _ks = keep_silent;
                 keep_silent = 1;
                 <Insert _obj SACK_OBJECT>;
@@ -372,6 +331,7 @@ else
                     rtrue;
                 }
             }
+		}
     }
     rfalse;
 ];
@@ -418,66 +378,114 @@ else
 	PrintShortName(p_obj);
 ];
 
-[ _PrintAfterEntry p_obj;
-	if(p_obj has container && P_obj hasnt open) print " (which is closed)";
-	if(p_obj has container && (p_obj has open || p_obj has transparent)) {
-		if(child(p_obj) == nothing)
-			print " (which is empty)";
-		else
-			if(PrintContents(" (which contains ", p_obj)) print ")";
-	}
-	if(p_obj has supporter && child(p_obj) ~= nothing) {
-		if(PrintContents(" (on which is ", p_obj)) print ")";
-	}
-!	if(p_obj has light && action == ##Inv) print " (providing light)";
-#Ifndef OPTIONAL_NO_DARKNESS;
-	if(p_obj has light) print " (providing light)";
-#Endif;
-	if(p_obj has worn && action == ##Inv) print " (worn)";
+[ _IsAreString p_plural;
+	if(p_plural == 2) return ARE_STR;
+	return IS_STR;
 ];
 
-[ PrintContents p_first_text p_obj p_check_workflag _obj _printed_first_text _printed_any_objects _last_obj _show_obj;
+[ _PrintAfterEntry p_obj;
+#Ifndef OPTIONAL_NO_DARKNESS;
+	if(p_obj has light && p_obj hasnt animate) print " (providing light)";
+#Endif;
+	if(p_obj has worn && action == ##Inv) print " (worn)";
+	if(p_obj has container && p_obj hasnt open) print " (which is closed)";
+	if(p_obj has container && (p_obj has open || p_obj has transparent)) {
+		if(PrintContents(1, p_obj) == 0) {
+			print " (which is empty)";
+		} else {
+			if(pc_indent == 0)
+				print " (which contains ";
+			else if(p_obj has open)
+				print " (which is open)";
+			PrintContents(0, p_obj);
+			if(pc_indent == 0) {
+				print (char) ')';
+			}
+		}
+
+	}
+	if(p_obj has supporter) {
+		if(pc_indent > 0)
+			PrintContents(0, p_obj);
+		else
+			if(PrintContents(" (on which ", p_obj, ISARE_BIT)) print (char) ')';
+	}
+];
+
+[ PrintContents p_first_text p_obj p_style
+		_obj _printed_any_objects _last_obj _show_obj _plural;
+! Print the contents of p_obj. Return true if anything was printed.
+! If any objects are printed, prefix with p_first_text.
+! If p_check_work_flag is true, only print objects which have workflag set.
+!   Special parameters:
+!   - If _p_first_text is a routine, it will be called with p_obj as argument
+!   - If p_first_text is 0, no prefix string will be printed
+!   - If p_first_text is 1, don't print anything, but return:
+!       0 if there are no printable objects in/on p_obj
+!       1 if there's exactly one printable object and it doesn't have pluralname
+!       2 if there are 2+ printable objects or one object with pluralname
+
+	if(p_first_text ~= 1) {
+		! This is a call to print something
+		if(pc_indent <= 0 && p_style & NEWLINE_BIT) { ! Non-recursive call with request to indent
+			pc_indent = 2;
+		} else if(pc_indent > 0)
+			pc_indent = pc_indent + 2;
+	}
+
 !   print "Objectlooping...^";
 	objectloop(_obj in p_obj) {
 !print "Considering ", (object) _obj, "...^";
-!if(_obj has concealed) print "Is concealed."; else print "Isnt concealed.";
 		_show_obj =
 			_obj ~= parent(player) && ! don't print container when player in it
-			(p_check_workflag == false || _obj has workflag);
+			(p_style & WORKFLAG_BIT == 0 || _obj has workflag);
 		if(action ~= ##Inv) {
 			! don't show concealed or scenery in the normal case (look etc.),
 			! but allow it when listing inventory.
 			if(_obj has concealed or scenery) _show_obj = false;
 		}
 
+		if(p_first_text == 1) {
+			if(_show_obj) {
+				if(_plural || _obj has pluralname) return 2;
+				_plural = 1;
+			}
+			continue;
+		}
+
 		if(_show_obj) {
-!			(_obj.&describe == 0 || _obj notin parent(player)) &&
-!			(_obj has moved || _obj.initial == 0 || _obj notin parent(player))) {
-			if(_printed_first_text == 0) {
+			if(_last_obj == 0) {
 				if(p_first_text ofclass String)
 					print (string) p_first_text;
 				else if(p_first_text ~= 0)
 					p_first_text(p_obj);
-				_printed_first_text = 1;
+				if(p_style & ISARE_BIT)
+					print (string) _IsAreString(PrintContents(1, p_obj));
 			}
 			! Push obj onto queue, printing the object that is shifted out, if any
 			if(_last_obj) {
-				if(_printed_any_objects) print ", ";
+				if(_printed_any_objects && pc_indent <= 0) print ", ";
 				_PrintContentsPrintAnObj(_last_obj);
 				_printed_any_objects = 1;
 			}
 			_last_obj = _obj;
 		}
 	}
-	if(_last_obj) {
-		if(_printed_any_objects) print " and ";
-		_PrintContentsPrintAnObj(_last_obj);
-	}
+	if(p_first_text == 1)
+		return _plural;
 
-	return _printed_first_text;
+	if(_last_obj) {
+		if(_printed_any_objects && pc_indent <= 0) print " and ";
+		_PrintContentsPrintAnObj(_last_obj);
+		_printed_any_objects = 1;
+	}
+	if(pc_indent > 0)
+		pc_indent = pc_indent - 2;
+	return _printed_any_objects;
 ];
 
 [ _PrintContentsPrintAnObj p_obj _inv _skip;
+	if(pc_indent > 0) { new_line; FastSpaces(pc_indent); }
 	if(p_obj.invent ~= 0) {
 		_inv = true;
 		inventory_stage = 1;
@@ -493,6 +501,22 @@ else
 	}
 ];
 
+[ FastSpaces p_spaces;
+#Ifv3;
+	while(p_spaces >= 6) {
+		print "      ";
+		p_spaces = p_spaces - 6;
+	}
+	for( : p_spaces > 0 : p_spaces--) @print_char ' ';
+#Ifnot;
+	while(p_spaces > 10) {
+		@print_table TenSpaces 10 1;
+		p_spaces = p_spaces - 10;
+	}
+	@print_table TenSpaces p_spaces 1;
+#Endif;
+];
+
 [ RunRoutines p_obj p_prop p_switch;
 #Ifndef OPTIONAL_NO_DARKNESS;
 	if(p_obj == thedark && p_prop ~= initial or short_name or description) p_obj = real_location;
@@ -517,32 +541,6 @@ else
 	}
 ];
 
-[ _InitObjects _i _k _stop;
-	_stop = top_object + 1;
-	for(_i = Directions : _i < _stop : _i++) {
-#Ifndef OPTIONAL_MANUAL_REACTIVE;
-#Ifdef OPTIONAL_REACTIVE_PARSE_NAME;
-		if(_i.&react_before ~= 0 || _i.&react_after ~= 0 || _i.&each_turn ~= 0 ||
-				_i.&add_to_scope ~= 0 || _i.&parse_name ~= 0)
-			give _i reactive;
-#Ifnot;
-		if(_i.&react_before ~= 0 || _i.&react_after ~= 0 || _i.&each_turn ~= 0 ||
-				_i.&add_to_scope ~= 0)
-			give _i reactive;
-#Endif;
-#Endif;
-		if(_i.&found_in) {
-#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
-			if(_k >= MAX_FLOATING_OBJECTS) {
-				RunTimeError(ERR_TOO_MANY_FLOATING);
-				rtrue;
-			}
-#EndIf;
-			floating_objects-->(_k++) = _i;
-		}
-	}
-];
-
 [ MoveFloatingObjects _i _j _len _obj _present;
 	while((_obj = floating_objects-->_i) ~= 0) {
 		_len = _obj.#found_in;
@@ -577,6 +575,8 @@ else
 			remove _obj;
 		_i++;
 	}
+	! It's not certain that scope has been modified, but PlayerTo relies on it
+	! being set.
 	scope_modified = true;
 ];
 
@@ -587,7 +587,7 @@ else
 	move Player to p_loc;
 	real_location = superparent(p_loc);
 	location = real_location;
-	MoveFloatingObjects(); ! Also sets scope_modified
+	MoveFloatingObjects(); ! Also sets scope_modified = true;
 #Ifndef OPTIONAL_NO_DARKNESS;
 	_UpdateDarkness();
 #Endif;
@@ -670,25 +670,39 @@ else
 ];
 #Endif;
 
+
+Include "scope.h";
 Include "parser.h";
 
 [ ActionPrimitive; indirect(#actions_table-->action); ];
 
-[ PerformPreparedAction;
+[ PerformPreparedAction _ret_val;
 #IfDef DEBUG;
 	if(debug_flag & 2) TraceAction();
 #EndIf;
 	if ((meta || (BeforeRoutines() == false)) && action < 4096) {
+		@push run_after_routines_msg; @push run_after_routines_arg_1;
+		run_after_routines_msg = 0;
 		ActionPrimitive();
-		return true; ! could run the command
+		! If the action has set run_after_routines_msg = true, after routines
+		! should be run. If it has set it to another value, this message should
+		! be printed unless after routines returns true.
+		if(run_after_routines_msg && AfterRoutines() == false &&
+				run_after_routines_msg ~= 0 or 1 &&
+				keep_silent == false)
+			PrintMsg(run_after_routines_msg, run_after_routines_arg_1);
+		@pull run_after_routines_arg_1; @pull run_after_routines_msg;
+		_ret_val = true; ! could run the command
 	}
-	return false; ! failed to run (stopped by BeforeRoutines)
+	return _ret_val;
 ];
 
 [ RunEachTurn _i _obj _scope_count _max;
 	! Run all each_turn routines for location and all objects in scope.
 #IfDef DEBUG;
+#IfV3;
 	if(debug_flag & 1 && location has reactive && location.&each_turn ~= 0) print "(", (name) location, ").each_turn()^";
+#EndIf;
 #EndIf;
 	_scope_count = GetScopeCopy();
 	RunRoutines(location, each_turn);
@@ -700,7 +714,9 @@ Include "parser.h";
 		_obj = scope_copy-->_i;
 		if(_obj has reactive && _obj.&each_turn ~= 0) {
 #IfDef DEBUG;
+#IfV3;
 			if(debug_flag & 1) print "(", (name) _obj, ").each_turn()^";
+#EndIf;
 #EndIf;
 			RunRoutines(_obj, each_turn);
 		}
@@ -730,13 +746,17 @@ Include "parser.h";
 	_scope_count = GetScopeCopy();
 #IfDef GamePreRoutine;
 #IfDef DEBUG;
+#IfV3;
 	if(debug_flag & 1) print "GamePreRoutine()^";
+#EndIf;
 #EndIf;
 	if(RunEntryPointRoutine(GamePreRoutine)) rtrue;
 #EndIf;
 
 #IfDef DEBUG;
+#IfV3;
 	if(debug_flag & 1) print "player.orders()^";
+#EndIf;
 #EndIf;
 	if(RunRoutines(player, orders)) rtrue;
 
@@ -746,7 +766,9 @@ Include "parser.h";
 		_obj = scope_copy-->_i;
 		if (_obj has reactive && _obj.&react_before ~= 0) {
 #IfDef DEBUG;
+#IfV3;
 			if(debug_flag & 1) print "(", (name) _obj, ").react_before()^";
+#EndIf;
 #EndIf;
 			if(RunRoutines(_obj, react_before)) {
 				rtrue;
@@ -755,14 +777,18 @@ Include "parser.h";
 		@inc_chk _i _max ?~next_entry;
 	}
 #IfDef DEBUG;
+#IfV3;
 	if(debug_flag & 1) print "(", (name) real_location, ").before()^";
+#EndIf;
 #EndIf;
 	if(real_location.&before) {
 		if(RunRoutines(real_location, before)) rtrue;
 	}
 	if(inp1 > 1) {
 #IfDef DEBUG;
+#IfV3;
 		if(debug_flag & 1) print "(", (name) inp1, ").before()^";
+#EndIf;
 #EndIf;
 		if(inp1.&before) {
 			if(RunRoutines(inp1, before)) rtrue;
@@ -782,21 +808,27 @@ Include "parser.h";
 		_obj = scope_copy-->_i;
 		if (_obj has reactive && _obj.&react_after ~= 0) {
 #IfDef DEBUG;
+#IfV3;
 			if(debug_flag & 1) print "(", (name) _obj, ").react_after()^";
+#EndIf;
 #EndIf;
 			if(RunRoutines(_obj, react_after)) rtrue;
 		}
 		@inc_chk _i _max ?~next_entry;
 	}
 #IfDef DEBUG;
+#IfV3;
 	if(debug_flag & 1) print "(", (name) real_location, ").after()^";
+#EndIf;
 #EndIf;
 	if(real_location.&after) {
 		if(RunRoutines(real_location, after)) rtrue;
 	}
 	if(inp1 > 1) {
 #IfDef DEBUG;
+#IfV3;
 		if(debug_flag & 1) print "(", (name) inp1, ").after()^";
+#EndIf;
 #EndIf;
 		if(inp1.&after) {
 			if(RunRoutines(inp1, after)) rtrue;
@@ -804,7 +836,9 @@ Include "parser.h";
 	}
 #IfDef GamePostRoutine;
 #IfDef DEBUG;
+#IfV3;
 	if(debug_flag & 1) print "GamePostRoutine()^";
+#EndIf;
 #EndIf;
 	if(RunEntryPointRoutine(GamePostRoutine)) rtrue;
 #EndIf;
@@ -813,27 +847,13 @@ Include "parser.h";
 
 [ RunLife p_actor p_reason;
 #IfDef DEBUG;
+#IfV3;
 	if(debug_flag & 1 && p_actor provides life) print "(", (name) p_actor, ").life()^";
+#EndIf;
 #EndIf;
     return RunRoutines(p_actor, life, p_reason);
 ];
 
-[ DirPropToFakeObj p_dir_prop;
-#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
-	if(p_dir_prop < N_TO_CONST || p_dir_prop > OUT_TO_CONST)
-		RunTimeError(ERR_NOT_DIR_PROP);
-#EndIf;
-	return p_dir_prop - N_TO_CONST + FAKE_N_OBJ;
-];
-
-[ FakeObjToDirProp p_fake_obj;
-#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
-	if(p_fake_obj < FAKE_N_OBJ || p_fake_obj > FAKE_OUT_OBJ)
-		RunTimeError(ERR_NOT_FAKE_OBJ);
-#EndIf;
-	return p_fake_obj - FAKE_N_OBJ + N_TO_CONST;
-];
-
 [ _SetDirectionIfIsFakeDir p_obj p_noun_no _idx;
 	if(p_obj >= FAKE_N_OBJ && p_obj <= FAKE_OUT_OBJ) {
 		_idx = p_obj - FAKE_N_OBJ;
@@ -1223,18 +1243,7 @@ Include "parser.h";
 ];
 #EndIf;
 
-#IfTrue RUNTIME_ERRORS < RTE_VERBOSE;
-[RT__Err err_no par1 par2;
-	print "Inform error: ";
-	if(err_no ofclass String)
-		print (string) err_no, " - ";
-	else
-		print_ret err_no;
-	print_ret " (", par1, ", ", par2, ")";
-];
-#EndIf;
-
-
+#Ifndef CUSTOM_PLAYER_OBJECT;
 Object selfobj "you"
 	with
 		name 'me' 'myself' 'self',
@@ -1252,6 +1261,7 @@ Object selfobj "you"
 		orders 0,
 		number 0,
 	has concealed animate proper transparent;
+#Endif;
 
 #Ifndef OPTIONAL_NO_DARKNESS;
 Object thedark "Darkness"
@@ -1288,7 +1298,9 @@ Object thedark "Darkness"
 [ EndTurnSequence;
 	turns++;
 #Ifndef STATUSLINE_SCORE;
+#Ifndef STATUSLINE_TIME;
 	if (sys_statusline_flag) {
+#Endif;
 		if (time_rate >= 0) the_time=the_time+time_rate;
 		else {
 			time_step--;
@@ -1298,7 +1310,9 @@ Object thedark "Darkness"
 			}
 		}
 		the_time = the_time % 1440;
+#Ifndef STATUSLINE_TIME;
 	}
+#Endif;
 #Endif;
 	RunTimersAndDaemons(); if(deadflag >= GS_DEAD) rtrue;
 	RunEachTurn(); if(deadflag >= GS_DEAD) rtrue;
@@ -1335,17 +1349,78 @@ Object thedark "Darkness"
 ];
 #Endif;
 
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+! Rarely used routines
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+#IfTrue RUNTIME_ERRORS < RTE_VERBOSE;
+[RT__Err err_no par1 par2;
+	print "Inform error: ";
+	if(err_no ofclass String)
+		print (string) err_no, " - ";
+	else
+		print_ret err_no;
+	print_ret " (", par1, ", ", par2, ")";
+];
+#EndIf;
+
+[ DirPropToFakeObj p_dir_prop;
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+	if(p_dir_prop < N_TO_CONST || p_dir_prop > OUT_TO_CONST)
+		RunTimeError(ERR_NOT_DIR_PROP);
+#EndIf;
+	return p_dir_prop - N_TO_CONST + FAKE_N_OBJ;
+];
+
+[ FakeObjToDirProp p_fake_obj;
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+	if(p_fake_obj < FAKE_N_OBJ || p_fake_obj > FAKE_OUT_OBJ)
+		RunTimeError(ERR_NOT_FAKE_OBJ);
+#EndIf;
+	return p_fake_obj - FAKE_N_OBJ + N_TO_CONST;
+];
+
+[ _InitObjects _i _k _stop;
+	_stop = top_object + 1;
+	for(_i = Directions : _i < _stop : _i++) {
+#Ifndef OPTIONAL_MANUAL_REACTIVE;
+#Ifdef OPTIONAL_REACTIVE_PARSE_NAME;
+		if(_i.&react_before ~= 0 || _i.&react_after ~= 0 || _i.&each_turn ~= 0 ||
+				_i.&add_to_scope ~= 0 || _i.&parse_name ~= 0)
+			give _i reactive;
+#Ifnot;
+		if(_i.&react_before ~= 0 || _i.&react_after ~= 0 || _i.&each_turn ~= 0 ||
+				_i.&add_to_scope ~= 0)
+			give _i reactive;
+#Endif;
+#Endif;
+		if(_i.&found_in) {
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+			if(_k >= MAX_FLOATING_OBJECTS) {
+				RunTimeError(ERR_TOO_MANY_FLOATING);
+				rtrue;
+			}
+#EndIf;
+			floating_objects-->(_k++) = _i;
+		}
+	}
+];
+
+[ SetTime p_time p_step;
+	the_time = p_time;
+	time_rate = p_step;
+	time_step = 0;
+	if(p_step < 0) time_step = -p_step;
+];
+
 #Ifdef NO_SCORE;
 [ main _i _j _copylength _sentencelength _parsearraylength _again_saved _parser_oops _disallow_complex_again;
 #Ifnot;
 [ main _i _j _copylength _sentencelength _parsearraylength _score _again_saved _parser_oops _disallow_complex_again;
 #Endif;
-#IfDef DEBUG;
 	dict_start = HDR_DICTIONARY-->0;
 	dict_entry_size = dict_start->(dict_start->0 + 1);
 	dict_start = dict_start + dict_start->0 + 4;
 	dict_end = dict_start + (dict_start - 2)-->0 * dict_entry_size;
-#EndIf;
 
 	parse->0 = MAX_INPUT_WORDS;
 #IfV5;
@@ -1357,7 +1432,11 @@ Object thedark "Darkness"
 	top_object = #largest_object-255;
 	sys_statusline_flag = ( ($1->0) & 2 ) / 2;
 
+#Ifdef CUSTOM_PLAYER_OBJECT;
+	player = CUSTOM_PLAYER_OBJECT;
+#Ifnot;
 	player = selfobj;
+#Endif;
 	deadflag = GS_PLAYING;
 	turns = -1;
 #Ifndef NO_SCORE;
@@ -1443,6 +1522,7 @@ Object thedark "Darkness"
 			if(_again_saved && _parser_oops > 0) {
 				_CopyInputArray(buffer2, buffer);
 				_CopyParseArray(parse2, parse);
+				num_words = parse -> 1;
 				_parser_oops-->0 = special_word;
 				_again_saved = false;
 				jump do_it_again;
@@ -1459,6 +1539,7 @@ Object thedark "Darkness"
 			} else if(_again_saved) {
 				_CopyInputArray(buffer2, buffer);
 				_CopyParseArray(parse2, parse);
+				num_words = parse -> 1;
 				jump do_it_again;
 			} else {
 				PrintMsg(MSG_PARSER_NOTHING_TO_AGAIN);
@@ -1478,7 +1559,7 @@ Object thedark "Darkness"
 			! to the end of the line to force new user input
 			_sentencelength = parse->1;
 		}
-		if(action >= 0 && meta == false) {
+		if(action >= 0 && meta == false && _sentencelength>0) {
 			EndTurnSequence();
         }
 
